/**
 * Copyright (C) 2009 STMicroelectronics
 *
 * This file is part of "Mind Compiler" is free software: you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: mind-members@lists.minalogic.net
 *
 * Authors: Matthieu Leclercq
 * Contributors: 
 */

package org.objectweb.fractal.mind.idl;

import org.objectweb.fractal.adl.xml.XMLNodeFactoryImpl;
import org.objectweb.fractal.mind.annotation.AnnotationChainFactory;
import org.objectweb.fractal.mind.idl.annotation.AnnotationLoader;
import org.objectweb.fractal.mind.idl.annotation.AnnotationProcessorLoader;
import org.objectweb.fractal.mind.idl.annotation.IDLLoaderPhase;
import org.objectweb.fractal.mind.idl.parser.IDLFileLoader;
import org.objectweb.fractal.mind.st.STNodeFactoryImpl;

public final class IDLLoaderChainFactory {

  private IDLLoaderChainFactory() {
  }

  public static IDLLocator newLocator() {
    IDLLocator idlLocator;
    final BasicIDLLocator bil = new BasicIDLLocator();
    idlLocator = bil;

    return idlLocator;
  }

  public static IDLLoader newLoader() {
    return newLoader(newLocator());
  }

  public static IDLLoader newLoader(final IDLLocator idlLocator) {

    // Loader chain components
    IDLLoader idlLoader;
    final IDLFileLoader ifl = new IDLFileLoader();
    final AnnotationLoader al = new AnnotationLoader();
    final AnnotationProcessorLoader apl1 = new AnnotationProcessorLoader();
    final IncludeLoader uil = new IncludeLoader();
    final ExtendsInterfaceLoader eil = new ExtendsInterfaceLoader();
    final KindDecorationLoader kdl = new KindDecorationLoader();
    final AnnotationProcessorLoader apl2 = new AnnotationProcessorLoader();
    final CacheIDLLoader cil = new CacheIDLLoader();

    idlLoader = cil;
    cil.clientIDLLoaderItf = apl2;
    apl2.clientIDLLoaderItf = kdl;
    kdl.clientIDLLoaderItf = uil;
    uil.clientIDLLoaderItf = eil;
    eil.clientIDLLoaderItf = apl1;
    apl1.clientIDLLoaderItf = al;
    al.clientIDLLoaderItf = ifl;

    apl1.setPhase(IDLLoaderPhase.AFTER_PARSING.name());
    apl2.setPhase(IDLLoaderPhase.AFTER_CHECKING.name());

    al.annotationCheckerItf = AnnotationChainFactory.newAnnotationChecker();

    // IncludeResolver sub-chain
    IncludeResolver includeResolver;
    final BasicIncludeResolver bir = new BasicIncludeResolver();
    final IncludeHeaderResolver ihr = new IncludeHeaderResolver();
    final InputResourcesIncludeResolver irir = new InputResourcesIncludeResolver();
    final CachingIncludeResolver cir = new CachingIncludeResolver();

    includeResolver = cir;
    cir.clientResolverItf = irir;
    irir.clientResolverItf = ihr;
    ihr.clientResolverItf = bir;

    bir.idlLoaderItf = idlLoader;
    cir.idlLoaderItf = idlLoader;

    uil.idlResolverItf = includeResolver;

    // Interface Reference Resolver
    InterfaceReferenceResolver interfaceReferenceResolver;
    final BasicInterfaceReferenceResolver birr = new BasicInterfaceReferenceResolver();
    final InputResourcesInterfaceReferenceResolver irirr = new InputResourcesInterfaceReferenceResolver();
    final ReferencedInterfaceResolver rir = new ReferencedInterfaceResolver();

    interfaceReferenceResolver = rir;
    rir.clientResolverItf = irirr;
    irirr.clientResolverItf = birr;
    birr.idlLoaderItf = idlLoader;

    eil.interfaceReferenceResolverItf = interfaceReferenceResolver;

    ifl.idlLocatorItf = idlLocator;

    // node factories
    final XMLNodeFactoryImpl xnf = new XMLNodeFactoryImpl();
    final STNodeFactoryImpl nf = new STNodeFactoryImpl();
    ifl.nodeFactoryItf = xnf;
    ihr.nodeFactoryItf = nf;

    return idlLoader;
  }
}
